@@ -30,8 +30,6 @@
 #include "gsd-automount-manager.h"
 #include "gsd-autorun.h"
 
-#define GSD_AUTOMOUNT_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_AUTOMOUNT_MANAGER, GsdAutomountManagerPrivate))
-
 #define GNOME_SESSION_DBUS_NAME      "org.gnome.SessionManager"
 #define GNOME_SESSION_DBUS_OBJECT    "/org/gnome/SessionManager"
 #define GNOME_SESSION_DBUS_INTERFACE "org.gnome.SessionManager"
@@ -52,12 +50,9 @@ struct GsdAutomountManagerPrivate
         GList *volume_queue;
 };
 
-static void     gsd_automount_manager_class_init  (GsdAutomountManagerClass *klass);
-static void     gsd_automount_manager_init        (GsdAutomountManager      *gsd_automount_manager);
+G_DEFINE_TYPE_WITH_PRIVATE (GsdAutomountManager, gsd_automount_manager, G_TYPE_OBJECT)
 
-G_DEFINE_TYPE (GsdAutomountManager, gsd_automount_manager, G_TYPE_OBJECT)
-
-GDBusProxy *
+static GDBusProxy *
 get_session_proxy (void)
 {
         static GDBusProxy *session_proxy;
@@ -514,7 +509,7 @@ setup_automounter (GsdAutomountManager *manager)
 				 manager, NULL);
 }
 
-void
+static void
 gsd_automount_manager_finalize (GObject *object)
 {
         GsdAutomountManager *manager  = GSD_AUTOMOUNT_MANAGER (object);
@@ -545,7 +540,7 @@ gsd_automount_manager_finalize (GObject *object)
 static void
 gsd_automount_manager_init (GsdAutomountManager *manager)
 {
-        manager->priv = GSD_AUTOMOUNT_MANAGER_GET_PRIVATE (manager);
+        manager->priv = gsd_automount_manager_get_instance_private (manager);
 
         g_debug ("Starting automounting manager");
 
@@ -559,8 +554,6 @@ gsd_automount_manager_class_init (GsdAutomountManagerClass *klass)
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
         object_class->finalize = gsd_automount_manager_finalize;
-
-        g_type_class_add_private (klass, sizeof (GsdAutomountManagerPrivate));
 }
 
 GsdAutomountManager *
