@@ -635,7 +635,9 @@ update_mouse_speed (GfInputSettings *settings,
 {
   gdouble value;
 
-  if (device && gdk_device_get_source (device) != GDK_SOURCE_MOUSE)
+  if (device &&
+      gdk_device_get_source (device) != GDK_SOURCE_MOUSE &&
+      gdk_device_get_source (device) != GDK_SOURCE_TRACKPOINT)
     return;
 
   value = g_settings_get_double (settings->mouse, "speed");
@@ -647,6 +649,7 @@ update_mouse_speed (GfInputSettings *settings,
   else
     {
       set_double_setting (settings, GDK_SOURCE_MOUSE, set_speed, value);
+      set_double_setting (settings, GDK_SOURCE_TRACKPOINT, set_speed, value);
     }
 }
 
@@ -656,7 +659,9 @@ update_mouse_natural_scroll (GfInputSettings *settings,
 {
   gboolean value;
 
-  if (device && gdk_device_get_source (device) != GDK_SOURCE_MOUSE)
+  if (device &&
+      gdk_device_get_source (device) != GDK_SOURCE_MOUSE &&
+      gdk_device_get_source (device) != GDK_SOURCE_TRACKPOINT)
     return;
 
   value = g_settings_get_boolean (settings->mouse, "natural-scroll");
@@ -669,6 +674,8 @@ update_mouse_natural_scroll (GfInputSettings *settings,
     {
       set_bool_setting (settings, GDK_SOURCE_MOUSE,
                         set_invert_scroll, value);
+      set_bool_setting (settings, GDK_SOURCE_TRACKPOINT,
+                        set_invert_scroll, value);
     }
 }
 
@@ -1212,6 +1219,13 @@ mapped_device_changed_cb (GSettings         *gsettings,
     update_device_display (info->settings, gsettings, info->device);
 }
 
+static void
+free_device_mapping_info (gpointer  data,
+                          GClosure *closure)
+{
+  g_free (data);
+}
+
 static void
 check_add_mappable_device (GfInputSettings *settings,
                            GdkDevice       *device)
@@ -1232,7 +1246,7 @@ check_add_mappable_device (GfInputSettings *settings,
 
   g_signal_connect_data (gsettings, "changed",
                          G_CALLBACK (mapped_device_changed_cb),
-                         info, (GClosureNotify) g_free, 0);
+                         info, free_device_mapping_info, 0);
 
   g_hash_table_insert (settings->mappable_devices, device, gsettings);
   update_device_display (settings, gsettings, device);
