@@ -48,6 +48,7 @@ typedef struct
 
   guint           timeout_id;
   gulong          changed_id;
+  gulong          closed_id;
 } GfBubblePrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GfBubble, gf_bubble, GF_TYPE_POPUP_WINDOW)
@@ -217,7 +218,7 @@ scale_pixbuf (GdkPixbuf *pixbuf,
   if (scale_factor_x > scale_factor_y)
     scale_factor = scale_factor_y;
 
-  if (scale_factor < 1.0)
+  if (scale_factor < 1.0f)
     {
       gint scale_x;
       gint scale_y;
@@ -272,6 +273,13 @@ add_timeout (GfBubble *bubble)
   priv->timeout_id = g_timeout_add (timeout, timeout_bubble, bubble);
 }
 
+static void
+destroy_widget (GtkWidget *widget,
+                gpointer   user_data)
+{
+  gtk_widget_destroy (widget);
+}
+
 static void
 update_bubble (GfBubble *bubble)
 {
@@ -306,7 +314,7 @@ update_bubble (GfBubble *bubble)
 
   body = nd_notification_get_body (priv->notification);
 
-  if (pango_parse_markup (body, -1, 0, NULL, NULL, NULL, NULL))
+  if (validate_markup (body))
     {
       gtk_label_set_markup (GTK_LABEL (priv->body_label), body);
     }
@@ -325,7 +333,7 @@ update_bubble (GfBubble *bubble)
 
   have_actions = FALSE;
   gtk_container_foreach (GTK_CONTAINER (priv->actions_box),
-                         (GtkCallback) gtk_widget_destroy, NULL);
+                         destroy_widget, NULL);
 
   actions = nd_notification_get_actions (priv->notification);
 
@@ -400,6 +408,15 @@ notification_changed_cb (NdNotification *notification,
   update_bubble (bubble);
 }
 
+static void
+notification_closed_cb (NdNotification *notification,
+                        gint            reason,
+                        GfBubble       *bubble)
+{
+  if (reason == ND_NOTIFICATION_CLOSED_API)
+    gtk_widget_destroy (GTK_WIDGET (bubble));
+}
+
 static void
 gf_bubble_dispose (GObject *object)
 {
@@ -421,6 +438,12 @@ gf_bubble_dispose (GObject *object)
       priv->changed_id = 0;
     }
 
+  if (priv->closed_id != 0)
+    {
+      g_signal_handler_disconnect (priv->notification, priv->closed_id);
+      priv->closed_id = 0;
+    }
+
   G_OBJECT_CLASS (gf_bubble_parent_class)->dispose (object);
 }
 
@@ -674,10 +697,15 @@ gf_bubble_new_for_notification (NdNotification *notification)
   priv = gf_bubble_get_instance_private (bubble);
 
   priv->notification = g_object_ref (notification);
+
   priv->changed_id = g_signal_connect (notification, "changed",
                                        G_CALLBACK (notification_changed_cb),
                                        bubble);
 
+  priv->closed_id = g_signal_connect (notification, "closed",
+                                      G_CALLBACK (notification_closed_cb),
+                                      bubble);
+
   update_bubble (bubble);
 
   return bubble;
