@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015 Alberts Muktupāvels
+ * Copyright (C) 2015-2017 Alberts Muktupāvels
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <X11/extensions/shape.h>
+#include <X11/extensions/Xfixes.h>
 #include <X11/Xatom.h>
 
 #include "gf-dbus-screenshot.h"
@@ -66,6 +67,186 @@ typedef enum
 
 G_DEFINE_TYPE (GfScreenshot, gf_screenshot, G_TYPE_OBJECT)
 
+static void
+free_pixels (guchar   *pixels,
+             gpointer  data)
+{
+  g_free (pixels);
+}
+
+static GdkPixbuf *
+pixels_to_pixbuf (gulong *pixels,
+                  gint    width,
+                  gint    height)
+{
+  guchar *data;
+  gint i, j;
+
+  data = g_new0 (guchar, width * height * 4);
+
+  for (i = j = 0; i < width * height; i++, j += 4)
+    {
+      guint32 argb;
+
+      argb = (guint32) pixels[i];
+      argb = (argb << 8) | (argb >> 24);
+
+      data[j] = argb >> 24;
+      data[j + 1] = (argb >> 16) & 0xff;
+      data[j + 2] = (argb >> 8) & 0xff;
+      data[j + 3] = argb & 0xff;
+    }
+
+  return gdk_pixbuf_new_from_data (data, GDK_COLORSPACE_RGB, TRUE, 8,
+                                   width, height, width * 4,
+                                   free_pixels, NULL);
+}
+
+static GdkPixbuf *
+get_cursor_pixbuf (GdkDisplay *display,
+                   gint       *xhot,
+                   gint       *yhot)
+{
+  Display *xdisplay;
+  gint event_base;
+  gint error_base;
+  XFixesCursorImage *image;
+  GdkPixbuf *pixbuf;
+
+  xdisplay = gdk_x11_display_get_xdisplay (display);
+
+  if (!XFixesQueryExtension (xdisplay, &event_base, &error_base))
+    return NULL;
+
+  image = XFixesGetCursorImage (xdisplay);
+
+  if (!image)
+    return NULL;
+
+  pixbuf = pixels_to_pixbuf (image->pixels, image->width, image->height);
+
+  *xhot = image->xhot;
+  *yhot = image->yhot;
+
+  XFree (image);
+
+  return pixbuf;
+}
+
+static gint
+get_window_scaling_factor (void)
+{
+  GValue gvalue = G_VALUE_INIT;
+  GdkScreen *screen;
+
+  g_value_init (&gvalue, G_TYPE_INT);
+
+  screen = gdk_screen_get_default ();
+  if (gdk_screen_get_setting (screen, "gdk-window-scaling-factor", &gvalue))
+    return g_value_get_int (&gvalue);
+
+  return 1;
+}
+
+static void
+screenshot_add_cursor (GdkPixbuf      *pixbuf,
+                       ScreenshotType  type,
+                       gboolean        include_cursor,
+                       GdkWindow      *window,
+                       gint            frame_offset_x,
+                       gint            frame_offset_y)
+{
+  GdkDisplay *display;
+  GdkPixbuf *cursor_pixbuf;
+  gint scale;
+  gint xhot;
+  gint yhot;
+
+  /* If we have a selected area, there were by definition no cursor
+   * in the screenshot.
+   */
+  if (!include_cursor || type == SCREENSHOT_AREA)
+    return;
+
+  display = gdk_display_get_default ();
+  cursor_pixbuf = get_cursor_pixbuf (display, &xhot, &yhot);
+  scale = get_window_scaling_factor ();
+
+  if (cursor_pixbuf == NULL)
+    {
+      GdkCursor *cursor;
+      cairo_surface_t *surface;
+      gdouble x_hot;
+      gdouble y_hot;
+      gint width;
+      gint height;
+
+      cursor = gdk_cursor_new_for_display (display, GDK_LEFT_PTR);
+
+      if (cursor == NULL)
+        return;
+
+      surface = gdk_cursor_get_surface (cursor, &x_hot, &y_hot);
+      g_object_unref (cursor);
+
+      if (surface == NULL)
+        return;
+
+      width = cairo_image_surface_get_width (surface);
+      height = cairo_image_surface_get_height (surface);
+
+      cursor_pixbuf = gdk_pixbuf_get_from_surface (surface, 0, 0, width, height);
+      cairo_surface_destroy (surface);
+
+      xhot = x_hot * scale;
+      yhot = y_hot * scale;
+    }
+
+  if (cursor_pixbuf != NULL)
+    {
+      GdkSeat *seat;
+      GdkDevice *device;
+      gdouble cx;
+      gdouble cy;
+      GdkRectangle pixbuf_rect;
+      GdkRectangle cursor_rect;
+
+      seat = gdk_display_get_default_seat (display);
+      device = gdk_seat_get_pointer (seat);
+
+      gdk_window_get_device_position_double (window, device, &cx, &cy, NULL);
+
+      pixbuf_rect.x = pixbuf_rect.y = 0;
+      pixbuf_rect.width = gdk_pixbuf_get_width (pixbuf);
+      pixbuf_rect.height = gdk_pixbuf_get_height (pixbuf);
+
+      cursor_rect.x = cx * scale - xhot - frame_offset_x;
+      cursor_rect.y = cy * scale - yhot - frame_offset_y;
+      cursor_rect.width = gdk_pixbuf_get_width (cursor_pixbuf);
+      cursor_rect.height = gdk_pixbuf_get_height (cursor_pixbuf);
+
+      /* see if the pointer is inside the window */
+      if (gdk_rectangle_intersect (&pixbuf_rect, &cursor_rect, &cursor_rect))
+        {
+          gint cursor_x;
+          gint cursor_y;
+
+          cursor_x = cx * scale - xhot - frame_offset_x;
+          cursor_y = cy * scale - yhot - frame_offset_y;
+
+          gdk_pixbuf_composite (cursor_pixbuf, pixbuf,
+                                cursor_rect.x, cursor_rect.y,
+                                cursor_rect.width, cursor_rect.height,
+                                cursor_x, cursor_y,
+                                1.0, 1.0,
+                                GDK_INTERP_BILINEAR,
+                                255);
+        }
+
+      g_object_unref (cursor_pixbuf);
+    }
+}
+
 static gchar *
 get_unique_path (const gchar *path,
                  const gchar *filename)
@@ -162,7 +343,7 @@ save_screenshot (GdkPixbuf    *pixbuf,
 }
 
 static void
-blank_rectangle_in_pixbuf (GdkPixbuf *pixbuf,
+blank_rectangle_in_pixbuf (GdkPixbuf    *pixbuf,
                            GdkRectangle *rect)
 {
   gint x;
@@ -237,25 +418,32 @@ blank_region_in_pixbuf (GdkPixbuf      *pixbuf,
 }
 
 static cairo_region_t *
-make_region_with_monitors (GdkScreen *screen)
+make_region_with_monitors (void)
 {
-  cairo_region_t *region;
+  GdkDisplay *display;
   gint num_monitors;
+  cairo_region_t *region;
   gint i;
 
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  num_monitors = gdk_screen_get_n_monitors (screen);
-  G_GNUC_END_IGNORE_DEPRECATIONS
-
+  display = gdk_display_get_default ();
+  num_monitors = gdk_display_get_n_monitors (display);
   region = cairo_region_create ();
 
   for (i = 0; i < num_monitors; i++)
     {
+      GdkMonitor *monitor;
+      gint scale;
       GdkRectangle rect;
 
-      G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      gdk_screen_get_monitor_geometry (screen, i, &rect);
-      G_GNUC_END_IGNORE_DEPRECATIONS
+      monitor = gdk_display_get_monitor (display, i);
+      scale = gdk_monitor_get_scale_factor (monitor);
+
+      gdk_monitor_get_geometry (monitor, &rect);
+
+      rect.x *= scale;
+      rect.y *= scale;
+      rect.width *= scale;
+      rect.height *= scale;
 
       cairo_region_union_rectangle (region, &rect);
     }
@@ -263,29 +451,36 @@ make_region_with_monitors (GdkScreen *screen)
   return region;
 }
 
+static void
+get_screen_size (gint *width,
+                 gint *height,
+                 gint  scale)
+{
+  GdkScreen *screen;
+  GdkWindow *window;
+
+  screen = gdk_screen_get_default ();
+  window = gdk_screen_get_root_window (screen);
+
+  *width = gdk_window_get_width (window) / scale;
+  *height = gdk_window_get_height (window) / scale;
+}
+
 static void
 mask_monitors (GdkPixbuf *pixbuf,
                GdkWindow *root_window)
 {
-  GdkScreen *screen;
-  cairo_region_t *region_with_monitors;
-  cairo_region_t *invisible_region;
   cairo_rectangle_int_t rect;
+  cairo_region_t *invisible_region;
+  cairo_region_t *region_with_monitors;
 
-  screen = gdk_window_get_screen (root_window);
-  region_with_monitors = make_region_with_monitors (screen);
-
-  rect.x = 0;
-  rect.y = 0;
-
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  rect.width = gdk_screen_get_width (screen);
-  rect.height = gdk_screen_get_height (screen);
-  G_GNUC_END_IGNORE_DEPRECATIONS
+  rect.x = rect.y = 0;
+  get_screen_size (&rect.width, &rect.height, 1);
 
   invisible_region = cairo_region_create_rectangle (&rect);
-  cairo_region_subtract (invisible_region, region_with_monitors);
+  region_with_monitors = make_region_with_monitors ();
 
+  cairo_region_subtract (invisible_region, region_with_monitors);
   blank_region_in_pixbuf (pixbuf, invisible_region);
 
   cairo_region_destroy (region_with_monitors);
@@ -347,6 +542,7 @@ get_gtk_frame_extents (GdkWindow *window,
   guchar *data;
   gint result;
   gulong *borders;
+  gint scale;
 
   display = gdk_display_get_default ();
   xdisplay = gdk_x11_display_get_xdisplay (display);
@@ -370,11 +566,12 @@ get_gtk_frame_extents (GdkWindow *window,
     }
 
   borders = (gulong *) data;
+  scale = get_window_scaling_factor ();
 
-  extents->left = borders[0];
-  extents->right = borders[1];
-  extents->top = borders[2];
-  extents->bottom = borders[3];
+  extents->left = borders[0] / scale;
+  extents->right = borders[1] / scale;
+  extents->top = borders[2] / scale;
+  extents->bottom = borders[3] / scale;
 
   XFree (data);
 
@@ -392,6 +589,7 @@ get_window_rect_coords (GdkWindow    *window,
   gint y;
   gint width;
   gint height;
+  gint scale;
   gint screen_width;
   gint screen_height;
 
@@ -427,18 +625,15 @@ get_window_rect_coords (GdkWindow    *window,
       y = 0;
     }
 
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  scale = get_window_scaling_factor ();
+  get_screen_size (&screen_width, &screen_height, scale);
 
-  screen_width = gdk_screen_width ();
   if (x + width > screen_width)
     width = screen_width - x;
 
-  screen_height = gdk_screen_height ();
   if (y + height > screen_height)
     height = screen_height - y;
 
-  G_GNUC_END_IGNORE_DEPRECATIONS
-
   if (screenshot_out != NULL)
     {
       screenshot_out->x = x;
@@ -471,15 +666,10 @@ static GdkWindow *
 find_active_window (void)
 {
   GdkScreen *screen;
-  GdkWindow *window;
 
   screen = gdk_screen_get_default ();
 
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  window = gdk_screen_get_active_window (screen);
-  G_GNUC_END_IGNORE_DEPRECATIONS
-
-  return window;
+  return gdk_screen_get_active_window (screen);
 }
 
 static GdkWindow *
@@ -545,6 +735,7 @@ take_screenshot_real (GfScreenshot    *screenshot,
 {
   GdkDisplay *display;
   GdkWindow *window;
+  gint scale;
   GtkBorder extents;
   GdkRectangle real;
   GdkRectangle s;
@@ -560,6 +751,8 @@ take_screenshot_real (GfScreenshot    *screenshot,
   if (window == NULL)
     return FALSE;
 
+  scale = get_window_scaling_factor ();
+
   if (type == SCREENSHOT_WINDOW && get_gtk_frame_extents (window, &extents))
     {
       gdk_window_get_frame_extents (window, &real);
@@ -580,6 +773,9 @@ take_screenshot_real (GfScreenshot    *screenshot,
       pixbuf = gdk_pixbuf_get_from_window (window, real.x, real.y,
                                            real.width, real.height);
 
+      screenshot_add_cursor (pixbuf, type, include_cursor, window,
+                             real.x * scale, real.y *scale);
+
       gdk_window_get_frame_extents (window, &real);
 
       *x = real.x;
@@ -656,7 +852,9 @@ take_screenshot_real (GfScreenshot    *screenshot,
 
           has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
 
-          tmp = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, s.width, s.height);
+          tmp = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
+                                s.width * scale, s.height * scale);
+
           gdk_pixbuf_fill (tmp, 0);
 
           for (i = 0; i < rectangle_count; i++)
@@ -665,41 +863,45 @@ take_screenshot_real (GfScreenshot    *screenshot,
               gint rec_y;
               gint rec_width;
               gint rec_height;
+              gint screen_width;
+              gint screen_height;
               gint y2;
 
               /* If we're using invisible borders, the ShapeBounding might not
                * have the same size as the frame extents, as it would include
                * the areas for the invisible borders themselves.
                * In that case, trim every rectangle we get by the offset between
-               * the WM window size and the frame extents. */
+               * the WM window size and the frame extents.
+               */
               rec_x = rectangles[i].x;
               rec_y = rectangles[i].y;
-              rec_width = rectangles[i].width - (frame_offset.left + frame_offset.right);
-              rec_height = rectangles[i].height - (frame_offset.top + frame_offset.bottom);
+              rec_width = rectangles[i].width;
+              rec_height = rectangles[i].height;
+
+              rec_width -= frame_offset.left * scale + frame_offset.right * scale;
+              rec_height -= frame_offset.top * scale + frame_offset.bottom * scale;
 
               if (real.x < 0)
                 {
-                  rec_x += real.x;
+                  rec_x += real.x * scale;
                   rec_x = MAX(rec_x, 0);
-                  rec_width += real.x;
+                  rec_width += real.x * scale;
                 }
 
               if (real.y < 0)
                 {
-                  rec_y += real.y;
+                  rec_y += real.y * scale;
                   rec_y = MAX(rec_y, 0);
-                  rec_height += real.y;
+                  rec_height += real.y * scale;
                 }
 
-              G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+              get_screen_size (&screen_width, &screen_height, 1);
 
-              if (s.x + rec_x + rec_width > gdk_screen_width ())
-                rec_width = gdk_screen_width () - s.x - rec_x;
+              if (s.x * scale + rec_x + rec_width > screen_width)
+                rec_width = screen_width - s.x * scale - rec_x;
 
-              if (s.y + rec_y + rec_height > gdk_screen_height ())
-                rec_height = gdk_screen_height () - s.y - rec_y;
-
-              G_GNUC_END_IGNORE_DEPRECATIONS
+              if (s.y * scale + rec_y + rec_height > screen_height)
+                rec_height = screen_height - s.y * scale - rec_y;
 
               for (y2 = rec_y; y2 < rec_y + rec_height; y2++)
                 {
@@ -735,65 +937,10 @@ take_screenshot_real (GfScreenshot    *screenshot,
         }
     }
 
-  /* If we have a selected area, there were by definition no cursor in the
-   * screenshot. */
-  if (include_cursor && type != SCREENSHOT_AREA)
-    {
-      GdkCursor *cursor;
-      GdkPixbuf *cursor_pixbuf;
-
-      cursor = gdk_cursor_new_for_display (display, GDK_LEFT_PTR);
-      cursor_pixbuf = gdk_cursor_get_image (cursor);
-
-      if (cursor_pixbuf != NULL)
-        {
-          GdkSeat *seat;
-          GdkDevice *device;
-          GdkRectangle rect;
-          gint cx;
-          gint cy;
-          gint xhot;
-          gint yhot;
-
-          seat = gdk_display_get_default_seat (display);
-          device = gdk_seat_get_pointer (seat);
-
-          if (wm_window != NULL)
-            gdk_window_get_device_position (wm_window, device, &cx, &cy, NULL);
-          else
-            gdk_window_get_device_position (window, device, &cx, &cy, NULL);
-
-          sscanf (gdk_pixbuf_get_option (cursor_pixbuf, "x_hot"), "%d", &xhot);
-          sscanf (gdk_pixbuf_get_option (cursor_pixbuf, "y_hot"), "%d", &yhot);
-
-          /* in rect we have the cursor window coordinates */
-          rect.x = cx + real.x;
-          rect.y = cy + real.y;
-          rect.width = gdk_pixbuf_get_width (cursor_pixbuf);
-          rect.height = gdk_pixbuf_get_height (cursor_pixbuf);
-
-          /* see if the pointer is inside the window */
-          if (gdk_rectangle_intersect (&real, &rect, &rect))
-            {
-              gint cursor_x;
-              gint cursor_y;
-
-              cursor_x = cx - xhot - frame_offset.left;
-              cursor_y = cy - yhot - frame_offset.top;
-
-              gdk_pixbuf_composite (cursor_pixbuf, pixbuf,
-                                    cursor_x, cursor_y,
-                                    rect.width, rect.height,
-                                    cursor_x, cursor_y,
-                                    1.0, 1.0,
-                                    GDK_INTERP_BILINEAR,
-                                    255);
-            }
-        }
-
-      g_clear_object (&cursor_pixbuf);
-      g_clear_object (&cursor);
-    }
+  screenshot_add_cursor (pixbuf, type, include_cursor,
+                         wm_window != NULL ? wm_window : window,
+                         frame_offset.left * scale,
+                         frame_offset.top * scale);
 
   if (type == SCREENSHOT_WINDOW)
     {
@@ -831,7 +978,7 @@ flashspot_data_new (GfScreenshot *screenshot,
 {
   FlashspotData *data;
 
-  data = (FlashspotData *) g_new0 (FlashspotData *, 1);
+  data = g_new0 (FlashspotData, 1);
 
   data->screenshot = g_object_ref (screenshot);
   data->sender = g_strdup (sender);
@@ -946,38 +1093,18 @@ take_screenshot (GfScreenshot          *screenshot,
   g_free (filename_out);
 }
 
-static void
-scale_area (gint *x,
-            gint *y,
-            gint *width,
-            gint *height)
-{
-}
-
-static void
-unscale_area (gint *x,
-              gint *y,
-              gint *width,
-              gint *height)
-{
-}
-
 static gboolean
 check_area (gint x,
             gint y,
             gint width,
             gint height)
 {
-  GdkScreen *screen;
+  gint scale;
   gint screen_width;
   gint screen_height;
 
-  screen = gdk_screen_get_default ();
-
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  screen_width = gdk_screen_get_width (screen);
-  screen_height = gdk_screen_get_height (screen);
-  G_GNUC_END_IGNORE_DEPRECATIONS
+  scale = get_window_scaling_factor ();
+  get_screen_size (&screen_width, &screen_height, scale);
 
   return x >= 0 && y >= 0 && width > 0 && height > 0 &&
          x + width <= screen_width && y + height <= screen_height;
@@ -992,17 +1119,14 @@ handle_screenshot (GfDBusScreenshot      *dbus_screenshot,
                    gpointer               user_data)
 {
   GfScreenshot *screenshot;
-  GdkScreen *screen;
+  gint scale;
   gint width;
   gint height;
 
   screenshot = GF_SCREENSHOT (user_data);
-  screen = gdk_screen_get_default ();
 
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  width = gdk_screen_get_width (screen);
-  height = gdk_screen_get_height (screen);
-  G_GNUC_END_IGNORE_DEPRECATIONS
+  scale = get_window_scaling_factor ();
+  get_screen_size (&width, &height, scale);
 
   take_screenshot (screenshot, invocation, SCREENSHOT_SCREEN,
                    FALSE, include_cursor, 0, 0, width, height,
@@ -1057,7 +1181,6 @@ handle_screenshot_area (GfDBusScreenshot      *dbus_screenshot,
       return TRUE;
     }
 
-  scale_area (&x, &y, &width, &height);
   take_screenshot (screenshot, invocation, SCREENSHOT_AREA,
                    FALSE, FALSE, x, y, width, height,
                    gf_dbus_screenshot_complete_screenshot_area,
@@ -1086,8 +1209,6 @@ handle_flash_area (GfDBusScreenshot      *dbus_screenshot,
       return TRUE;
     }
 
-  scale_area (&x, &y, &width, &height);
-
   flashspot = gf_flashspot_new ();
   gf_flashspot_fire (flashspot, x, y, width, height);
   g_object_unref (flashspot);
@@ -1107,13 +1228,23 @@ handle_select_area (GfDBusScreenshot      *dbus_screenshot,
   gint y;
   gint width;
   gint height;
+  gboolean selected;
 
   select_area = gf_select_area_new ();
   x = y = width = height = 0;
 
-  if (gf_select_area_select (select_area, &x, &y, &width, &height))
+  selected = gf_select_area_select (select_area, &x, &y, &width, &height);
+  g_object_unref (select_area);
+
+  gdk_flush ();
+
+  if (selected)
     {
-      unscale_area (&x, &y, &width, &height);
+      /* wait 200ms to allow compositor finish redrawing selected area
+       * without selection overlay.
+       */
+      g_usleep (G_USEC_PER_SEC / 5);
+
       gf_dbus_screenshot_complete_select_area (dbus_screenshot, invocation,
                                                x, y, width, height);
     }
@@ -1124,8 +1255,6 @@ handle_select_area (GfDBusScreenshot      *dbus_screenshot,
                                              "Operation was cancelled");
     }
 
-  g_object_unref (select_area);
-
   return TRUE;
 }
 
