@@ -32,8 +32,6 @@
 #include "gvc-channel-bar.h"
 #include "gvc-stream-status-icon.h"
 
-#define GVC_STREAM_STATUS_ICON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GVC_TYPE_STREAM_STATUS_ICON, GvcStreamStatusIconPrivate))
-
 struct GvcStreamStatusIconPrivate
 {
         char          **icon_names;
@@ -55,7 +53,7 @@ enum
 
 static void     gvc_stream_status_icon_finalize   (GObject                  *object);
 
-G_DEFINE_TYPE (GvcStreamStatusIcon, gvc_stream_status_icon, GTK_TYPE_STATUS_ICON)
+G_DEFINE_TYPE_WITH_PRIVATE (GvcStreamStatusIcon, gvc_stream_status_icon, GTK_TYPE_STATUS_ICON)
 
 static void
 on_adjustment_value_changed (GtkAdjustment *adjustment,
@@ -111,6 +109,13 @@ ungrab (GvcStreamStatusIcon *icon,
 	gtk_widget_hide (icon->priv->dock);
 }
 
+static void
+show_all_cb (GtkWidget *widget,
+             gpointer   user_data)
+{
+  gtk_widget_show_all (widget);
+}
+
 static gboolean
 popup_dock (GvcStreamStatusIcon *icon,
             guint                time)
@@ -150,13 +155,11 @@ popup_dock (GvcStreamStatusIcon *icon,
         gvc_channel_bar_set_orientation (GVC_CHANNEL_BAR (icon->priv->bar),
                                          1 - orientation);
 
-        G_GNUC_BEGIN_IGNORE_DEPRECATIONS
         monitor_num = gdk_screen_get_monitor_at_point (screen, area.x, area.y);
         gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
-        G_GNUC_END_IGNORE_DEPRECATIONS
 
         gtk_container_foreach (GTK_CONTAINER (icon->priv->dock),
-                               (GtkCallback) gtk_widget_show_all, NULL);
+                               show_all_cb, NULL);
         gtk_widget_get_preferred_size (icon->priv->dock, &dock_req, NULL);
 
         if (orientation == GTK_ORIENTATION_VERTICAL) {
@@ -424,20 +427,20 @@ update_icon (GvcStreamStatusIcon *icon)
                 markup = g_strdup_printf (
                                           "<b>%s: %.0f%%</b>\n<small>%0.2f dB\n%s</small>",
                                           icon->priv->display_name,
-                                          100 * (float)volume / PA_VOLUME_NORM,
+                                          100 * (double) volume / PA_VOLUME_NORM,
                                           db,
                                           gvc_mixer_stream_get_description (icon->priv->mixer_stream));
         } else if (can_decibel) {
                 markup = g_strdup_printf (
                                           "<b>%s: %.0f%%</b>\n<small>-&#8734; dB\n%s</small>",
                                           icon->priv->display_name,
-                                          100 * (float)volume / PA_VOLUME_NORM,
+                                          100 * (double) volume / PA_VOLUME_NORM,
                                           gvc_mixer_stream_get_description (icon->priv->mixer_stream));
         } else {
                 markup = g_strdup_printf (
                                           "<b>%s: %.0f%%</b>\n<small>%s</small>",
                                           icon->priv->display_name,
-                                          100 * (float)volume / PA_VOLUME_NORM,
+                                          100 * (double) volume / PA_VOLUME_NORM,
                                           gvc_mixer_stream_get_description (icon->priv->mixer_stream));
         }
 
@@ -726,8 +729,6 @@ gvc_stream_status_icon_class_init (GvcStreamStatusIconClass *klass)
                                                              "Name of icon to display for this stream",
                                                               G_TYPE_STRV,
                                                               G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
-
-        g_type_class_add_private (klass, sizeof (GvcStreamStatusIconPrivate));
 }
 
 static void
@@ -746,7 +747,7 @@ on_status_icon_visible_notify (GvcStreamStatusIcon *icon)
 static void
 gvc_stream_status_icon_init (GvcStreamStatusIcon *icon)
 {
-        icon->priv = GVC_STREAM_STATUS_ICON_GET_PRIVATE (icon);
+        icon->priv = gvc_stream_status_icon_get_instance_private (icon);
 
         g_signal_connect (icon,
                           "activate",
